﻿using FlowerShopContracts.BindingModels;
using FlowerShopContracts.BusinessLogicsContracts;
using FlowerShopContracts.SearchModels;
using FlowerShopDataModels.Models;
using Microsoft.Extensions.Logging;

namespace FlowerShopView
{
	public partial class FormShop : Form
	{
		private readonly ILogger _logger;

		private readonly IShopLogic _logic;

		private int? _id;

		private Dictionary<int, (IFlowerModel, int)> _shopFlowers;

		public int Id { set { _id = value; } }

		public FormShop(ILogger<FormShop> logger, IShopLogic logic)
		{
			InitializeComponent();
			_logger = logger;
			_logic = logic;
			_shopFlowers = new Dictionary<int, (IFlowerModel, int)>();
		}

		private void FormShop_Load(object sender, EventArgs e)
		{
			if (_id.HasValue)
			{
				_logger.LogInformation("Shop loading");
				try
				{
					var view = _logic.ReadElement(new ShopSearchModel { Id = _id.Value });
					if (view != null)
					{
						textBoxName.Text = view.ShopName;
						textBoxAddress.Text = view.Address;
						dateTimePicker.Value = view.DateOpening;
						_shopFlowers = view.ShopFlowers ?? new Dictionary<int, (IFlowerModel, int)>();
						LoadData();
					}
				}
				catch (Exception ex)
				{
					_logger.LogError(ex, "Shop loading error");
					MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
				}
			}
		}

		private void LoadData()
		{
			_logger.LogInformation("Shop flowers loading");
			try
			{
				if (_shopFlowers != null)
				{
					dataGridView.Rows.Clear();
					foreach (var iceCream in _shopFlowers)
					{
						dataGridView.Rows.Add(new object[] { iceCream.Key, iceCream.Value.Item1.FlowerName, iceCream.Value.Item2 });
					}
				}
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Shop flowers loading error");
				MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void ButtonSave_Click(object sender, EventArgs e)
		{
			if (string.IsNullOrEmpty(textBoxName.Text))
			{
				MessageBox.Show("Заполните название", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			if (string.IsNullOrEmpty(textBoxAddress.Text))
			{
				MessageBox.Show("Заполните адрес", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			if (string.IsNullOrEmpty(dateTimePicker.Text))
			{
				MessageBox.Show("Заполните дату", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			_logger.LogInformation("Shop saving");
			try
			{
				var model = new ShopBindingModel
				{
					Id = _id ?? 0,
					ShopName = textBoxName.Text,
					Address = textBoxAddress.Text,
					DateOpening = dateTimePicker.Value,
					ShopFlowers = _shopFlowers,
					MaximumFlowers = Convert.ToInt32(textBoxMaximum.Text),
				};
				var operationResult = _id.HasValue ? _logic.Update(model) : _logic.Create(model);
				if (!operationResult)
				{
					throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
				}
				MessageBox.Show("Сохранение прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
				DialogResult = DialogResult.OK;
				Close();
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Shop saving error");
				MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void ButtonCancel_Click(object sender, EventArgs e)
		{
			DialogResult = DialogResult.Cancel;
			Close();
		}
	}
}
